#include <stdio.h>
#include <search.h>
#include "util.h"
#include "color.h" /* valid_colors[]; */

extern void check_safechars(const char* s);  /* XXX - should go in util */

enum { CMAP_MAX=1024 };
typedef struct { char *name, *color; } Cmap;
static Cmap cmap[CMAP_MAX];
static int ncmap = 0;

/* the underlying real colors that we can alias */
void check_basic_color(const char* s)
{
    int i;
    check_safechars(s);
    for (i=0; valid_colors[i]; ++i)
	if (same_string(s, valid_colors[i]))
	    return;
    fatalf("the color {%s} is not defined", s);
}

int cmap_cmp(const void *a, const void *b)
{
    return strcmp(((Cmap*)a)->name, ((Cmap*)b)->name);
} 

void cmap_init()
{
    int i;
    Cmap elt;
    for (i = 0; valid_colors[i]; ++i) {
	elt.name = elt.color = valid_colors[i];
	if (ncmap >= CMAP_MAX) { fatal("cmap full"); }
	if (lsearch(&elt, cmap, &ncmap, sizeof(elt), cmap_cmp) == 0)
	    fatalf("cannot initialize cmap (elt #%d)\n", i);
    }
    /* fprintf(stderr, "cmap init %d colors (%d)\n", i, ncmap); */
}

/* add name for color */
void cmap_add(char* name, char* color)
{
    Cmap t, *p;

    if (ncmap >= CMAP_MAX) { fatal("cmap full"); }
    check_basic_color(color);
    t.name = copy_string(name); t.color = copy_string(color);
    /* fprintf(stderr, "cmap adding name(%s) -> color(%s) (%d colors)\n", name, color, ncmap); */
    if ((p = lsearch(&t, cmap, &ncmap, sizeof(t), cmap_cmp)) == 0)
	fatalf("cannot add to cmap name(%s) -> color(%s)\n", name, color);
    /* fprintf(stderr, "cmap added name(%s) -> color(%s) (%d colors)\n", name, color, ncmap); */
}

/* get color from name */
char *cmap_get(char* name)
{
    Cmap t, *p;
    t.name = name; t.color = 0;
    /* fprintf(stderr, "cmap looking (%s)\n", name); */
    if ((p = lfind(&t, cmap, &ncmap, sizeof(t), cmap_cmp)) == 0)
	fatalf("cannot find in cmap (%s)\n", name);
    /* fprintf(stderr, "cmap found name(%s) -> color(%s) (%d)\n", p->name, p->color, ncmap); */
    return p->color;
}
