/* CpG.c -- make tags for CpG islands, i.e., regions of at least 50% G+C where
	the ratio of CpG dinucleotides relative to GpC is at least a
	specified percent.
*/

#include "util.h"
#include "seq.h"

enum {
  WINDOW=200,
  SHIFT=100
};

static void record(int from, int to, int offset, int *from_addr, int *to_addr,
	char *tag)
{
	if (*to_addr >= 0 && *to_addr < from) {
		printf("%d %d %s\n", *from_addr+offset, *to_addr+offset, tag);
		*from_addr = *to_addr = -1;
	}
	if (*from_addr == -1)
		*from_addr = from;
	*to_addr = to;
}

int main(int argc, char **argv)
{
	int i, from, to, nGC, nCpG, nGpC, len;
	int seg_len, pct, min_pct, offset, from_save, to_save;
	uchar *seq;
	char c, cc;
	SEQ *sp;

	argv0 = "CpG";
	if (argc != 4)
		fatal("args: seq_file min_pct tag");

	if ((min_pct = atoi(argv[2])) <= 10)
		fatal("min percent CpG/GpC must exceed 10.");

	sp = seq_get(argv[1]);
	len = SEQ_LEN(sp);
        seq = SEQ_CHARS(sp);
	offset = SEQ_FROM(sp);
	--offset; /* sequence positions start at 1 */
	from_save = to_save = -1; /* remembered interval set to empty */
	for (from = to = 0; to < len; from += SHIFT) {
		to = from + WINDOW;
		if (to + SHIFT >= len)
			to = len;
		seg_len = to - from;
		nGpC = nCpG = nGC = 0;
		for (i = from; i < to; ++i) {
			c = seq[i];
			cc = (i > 0 ? seq[i-1] : '\0');
			if (c == 'G' || c == 'C')
				++nGC;
			if (cc == 'C' && c == 'G')
				++nCpG;
			if (cc == 'G' && c == 'C')
				++nGpC;
		}
		pct = (nGpC > 0 ? (100*nCpG)/nGpC : 100*nCpG);
		if (2*nGC >= seg_len && pct >= min_pct)
			record(from, to, offset, &from_save, &to_save, argv[3]);
	}
	seq_close(sp);

	/* flush the last CpG island */
	record(len+1, len+1, offset, &from_save, &to_save, argv[3]);

	return 0;
}
