/* contigs - split a sequence file into contigs at NNN..NN */

#include "util.h"
#include "seq.h"

enum {
 LINE_LEN=50,
 DEFAULT_MIN_N=90,
 BUF_LEN=5000,
 NAM_MAX=500000,
};

int pr_bp(uchar c, int col)
{
	putchar(c);
	if (++col == LINE_LEN) {
		putchar('\n');
		col = 0;
	}
	return col;
}

uchar *nam[NAM_MAX];
int main(int argc, char **argv)
{
	int N, contig, i, j, col, n, min_N = DEFAULT_MIN_N;
	char *header, buf[BUF_LEN+1];
	uchar *seq;
	SEQ *sf;

	argv0 = "contigs";
	if (argc != 2 && argc != 3)
		fatalf("args: seq_file [min_N]", argv[0]);

	if (argc == 3 && (min_N = atoi(argv[2])) <= 0)
		fatal("min_N must be positive");
	sf = seq_open(argv[1]);
	n = -1;
	while (seq_read(sf)) {
		++n;
		if (n == NAM_MAX)
			fatalf("too many contigs (%d)\n", n);
		header = SEQ_HEAD(sf);
		nam[n] = fasta_name(header);
		for (i = 0; i < n; ++i)
			if (same_string(nam[n], nam[i]))
				fatalf("duplicate name: '%s'", nam[n]);
		strcpy(buf, nam[n]);
		if (strlen(header) > 0)
			strcat(buf, ".");

		N = SEQ_LEN(sf);
		seq = SEQ_CHARS(sf);
		if (strlen(header) > 0)
			printf("%s\n", header);
		else
			printf(">C1\n");
		contig = 1;
		for (i = col =  0; i < N; ++i) {
			if (seq[i] == 'N') {
				for (j = i+1; j < N && seq[j] == 'N'; ++j)
					;
				if (i > 0 && j < N && j >= i + min_N) {
					if (col != 0)
						putchar('\n');
					printf(">%sC%d\n", buf, ++contig);
					col = 0;
					i = j;
				} else
					while (i < j)
						col = pr_bp(seq[i++], col);
			}
			if (i < N)
				col = pr_bp(seq[i], col);
		}
		if (col != 0)
			putchar('\n');
	}

	seq_close(sf);
	return 0;
	}
