/* dottag1.c -- make a dotplot tag file for sequence 1
*
* syntax:
*	dottag1 exons-file blastz-file
*/

#include "util.h"
#include "exons.h"
#include "piplib.h"
#include "lav.h"
#include "args.h"
#include "genedir.h"

#define DEFAULT_NSEQ 2

int main(int argc, char **argv)
{
	char **gene_name, **exon_name;
	int nexons, i, j, k, n, *beg, *end, *dir, ngene, last_label, min_sep,
		g, mid, *gene, S;

	argv0 = "dottag1";
	if (argc < 3)
		fatal("args = exons-file blastz-file");
	ckargs("S", argc, argv, 2);
	get_argval_pos('S', &S, DEFAULT_NSEQ);

	min_sep = dotclash(argv[2], S);

	nexons = get_exons(argv[1], &beg, &end, &dir, &gene_name,
	  &exon_name, 0);

	for (ngene = n = 0; n < nexons; ++n)
		if (gene_name[n] != NULL)
			++ngene;
	if (ngene == 0) {
		printf("  [[(first sequence) /Times-Roman]] Banner\n");
		exit(0);
	}
	gene = (int *)ckalloc(ngene*sizeof(int));
	for (i = n = 0; n < nexons; ++n)
		if (gene_name[n] != NULL) {
			quote_for_ps(gene_name[n]);
			gene[i++] = n;
		}

	/* bubble sort */
	for (i = 0; i < ngene; ++i) {
		k = i;
		for (j = i+1; j < ngene; ++j)
			if (beg[gene[j]] < beg[gene[k]])
				k = j;
		n = gene[i];
		gene[i] = gene[k];
		gene[k] = n;
	}

	tag_header();

	last_label  = -99999999;
	for (n = 0; n < ngene; ++n) {
		g = gene[n];
		printf("%d %d ", beg[g], end[g]);
		mid = (beg[g]+end[g])/2;
		if (mid - last_label >= min_sep) {
			printf("[[(%s) /Times-Roman]]", gene_name[g]);
			last_label = mid;
		} else
			printf("Null");
		printf(" %s Black Arrow\n", genedirstr(dir[g]));
	}
	free(gene);
	free_exons(beg, end, dir, gene_name, exon_name, nexons);
	return 0;
}
