/*
* dottag2.c - prepare a dotplot tag file for sequence 2
*
* syntax:
*       dottag2 blastz-output
*/

#include "util.h"
#include "args.h"
#include "lav.h"
#include "piplib.h"

#define DEFAULT_NSEQ 2

enum {
  BUF_SIZE=500,MAX_NAME=15
};

int main(int argc, char **argv)
{
	FILE *fp;
	int P1, L2, L2tot;
	int P1old, L2old;
	int rc, nth, nth_old;
	int last_label, min_sep, mid, S;
	char buf[BUF_SIZE], name[BUF_SIZE], contig_name[BUF_SIZE], *s;

	argv0 = "dottag2";
	if (argc != 2)
		fatal("arg = blastz-output");

	ckargs("S", argc, argv, 2);
	get_argval_pos('S', &S, DEFAULT_NSEQ);

	min_sep = dotclash(argv[1], S);

	fp = ckopen(argv[1], "r");

	tag_header();
	
	last_label = -99999999;
	L2tot = 0;
	buf[0] = '\0';
	P1old = L2old = 0;
	nth_old = 0;
	while (lav_seqname4(2, fp, buf, BUF_SIZE, name, &P1, &L2, &rc, &nth))
		if (nth != nth_old) {
			P1old = P1; L2old = L2; nth_old = nth;
			(void) lav_contigname(fp, buf, BUF_SIZE, contig_name);
			if ((s = strchr(contig_name, '(')) != NULL)
				*s = '\0';
			if (strlen(contig_name) > MAX_NAME) {
				contig_name[MAX_NAME] = '~';
				contig_name[MAX_NAME+1] = '\0';
			}
			if (contig_name[0] == '\0')
				strcpy(contig_name, " sequence 2");
			quote_for_ps(contig_name);
			printf("%d %d ", L2tot, L2tot + L2);
			mid = L2tot + L2/2;
			if (mid - last_label >= min_sep) {
				printf("[[(%s  ) /Times-Roman]]",
					contig_name+1); /* skip '>' */
				last_label = mid;
			} else
				printf("Null");
			printf(" Right White Arrow\n");
			L2tot += L2;
		}

	fclose(fp);
	return 0;
}
