/*
* order_contigs.c - use predicted order and orientation to shuffle contigs
*
* Syntax: order_contigs OandO-output contigs-file
*
*/

enum {
  MAX_CONTIGS=10*1000*1000,
  BUF_SIZE=500,
  LINE_LEN=50
};

#include "util.h"
#include "seq.h"

typedef struct cont {
	char *header;
	uchar *seq;
	int len, found;
} Contig;

Contig C[MAX_CONTIGS];

int pr_bp(uchar c, int col)
{
	putchar(c);
	if (++col == LINE_LEN) {
		putchar('\n');
		col = 0;
	}
	return col;
}

int main(int argc, char **argv)
{
	SEQ *sf;
	FILE *fp;
	char buf[500], *s, *t;
	int i, j, col, nc, rc, seen;

	argv0 = "order_contigs";
	if (argc != 3)
		fatal("args = OandO-output sequence-file");

	/* read the contigs */
	sf = seq_open(argv[2]);
	for (nc = 0; seq_read(sf); ++nc) {
		if (nc >= MAX_CONTIGS)
			fatal("Too many contigs.");
		C[nc].header = copy_string(SEQ_HEAD(sf));
		C[nc].seq = copy_string(SEQ_CHARS(sf));
		C[nc].len = SEQ_LEN(sf);
		C[nc].found = 0;
	}
	seq_close(sf);
/*
for (i = 0; i < nc; ++i)
printf("%d: %d %s\n", i, C[i].len, C[i].name);
*/


	fp = ckopen(argv[1], "r");
	while (fgets(buf, BUF_SIZE, fp)) {
		if ((s = strchr(buf, '>')) == NULL)
			fatalf("no '>': %s", buf);
		if ((t = strchr(buf, '\n')) != NULL)
			*t = '\0';
		if (buf[strlen(buf)-1] == '-') {
			rc = 1;
			buf[strlen(buf)-1] = '\0';
		} else
			rc = 0;
		seen = -1;
		for (i = 0; i < nc; ++i)
			if (same_string(s, C[i].header)) {
				if (seen > -1)
					fatalf("Duplicate contig names '%s'\n",
					  s);
				seen = i;
			}
		if (seen == -1)
			fatalf("Cannot find contig '%s'\n", s);
		C[seen].found = 1;
		printf("%s%s\n", C[seen].header, rc ? " (rev. comp.)":"");
		if (rc)
			do_revcomp(C[seen].seq, C[seen].len);
		for (j = col =  0; j < C[seen].len; ++j) {
			col = pr_bp(C[seen].seq[j], col);
		}
		if (col != 0)
			putchar('\n');
	}
	for (i = 0; i < nc; ++i)
		if (!C[i].found)
			fprintf(stderr, "Did not find %s\n", C[i].header);
	fclose(fp);

	return 0;
}
