#RCSID='$Id: make-pip.mk,v 1.18 2003/02/10 20:47:10 schwartz Exp $'
STYLE=${PIPHOME}/lib/tag.style100
#STYLE=/usr/local/align/pipmaker/lib/tag.style100

input_files: seq1data seq1mask seq2data exons colors email \
	format strand chain genpip gendot genono genpdf gentxt \
	genlat genxan genlav piptitle novice gentig

input_env:
	@echo PIPHOME=$${PIPHOME:?}
	@echo STYLE=${STYLE}
	@echo TXT_MAX=${TXT_MAX}

%.pdf: %.ps
	pip-distill $@

out.style: $(STYLE)
	cp $(STYLE) $@

out.lat: out.blastz novice
	pip-aux-mklat

out.pp: out.blastz
	concise-align out.blastz >$@

out.blastz: out.seq1m seq2data chain strand sensitivity
	if [ -s ualign ]; then\
	  lav-filter <ualign >out.blastz;\
	else\
	  pip-aux-blastz out.seq1m seq2data chain strand sensitivity >$@ ;\
	fi

out.ono: out.ono.blastz
	single_cov out.ono.blastz > out.ono.single
	OandO out.ono.single >$@

out.tig2: seq2data
	set -x; if f.bool ./gentig; \
	then contigs seq2data > $@ ; \
	else ln seq2data $@ ; \
	fi

out.ono.blastz: out.seq1m out.tig2 chain strand sensitivity
	pip-aux-blastz out.seq1m out.tig2 chain strand sensitivity >$@
	ln $@ out.blastz

out.ono.ps: out.ono out.ono.blastz out.undr out.style out.tig2
	order_lav out.ono out.ono.blastz > out.order
	mkdotplot out.order exons out.undr out.style out.tig2 > $@

out.ono.seq: out.ono out.tig2
	order_contigs out.ono out.tig2 > $@

out.tags_0: exons seq1data out.repeats
	gene_tag exons  >$@
	cat out.repeats >>$@
	CpG seq1data 60 CpG60 >>$@
	CpG seq1data 75 CpG75 >>$@

# optional user input
annotations:
	touch $@

out.bookmrk: exons annotations
	exon-bookmarks exons >$@
	pip-annot -b annotations >>$@

out.tags: out.tags_0 annotations out.bookmrk
	group_tags out.tags_0 >$@
	pip-annot annotations >>$@

out.legend.tr: annotations  out.title_ps colors
	mklegend2 "`cat out.title_ps`" >out.p.legend
	annot-legend annotations >out.a.legend
	underlays-legend colors >out.u.legend
	pip-annot -t annotations >$@
	echo ".PSPIC out.p.legend 8.5i" >>$@
	echo ".PSPIC out.a.legend 8.5i" >>$@
	echo ".PSPIC out.u.legend 8.5i" >>$@

# note: requires groff 1.16 or better
out.legend.ps: out.legend.tr
	groff -S out.legend.tr >>$@

out.pip.mpip: out.tags out.style out.undr out.blastz
	mpip -h out.style -t out.tags -u out.undr -a out.blastz -s '' -s '' >$@

out.pip.ps: out.ov.ps out.pip.mpip out.legend.ps out.bookmrk
	echo '%!PS-Adobe-3.0' >$@
	echo '%%Orientation: Portrait' >>$@
	cat out.ov.ps out.pip.mpip out.legend.ps out.bookmrk >>$@
	echo 'showpage' >>$@

out.exons: exons out.blastz
	analyze_exons exons out.blastz >$@

out.dot.ps: out.blastz exons out.undr out.style
	mkdotplot out.blastz exons out.undr out.style seq2data >$@

out.title_ps: piptitle
	pip-aux-mktitle piptitle exons >$@

out.repeats: seq1mask
	pip-aux-repeats seq1mask >$@

out.seq1m: out.repeats seq1data
	seqmask seq1data out.repeats >$@

out.undr_0: colors
	mkunderlays colors >>$@

out.undr: out.undr_0 out.blastz
	[ "`f.int ./genpdf`" -lt 2 ] || mkunderlays2 out.blastz >$@
	cat out.undr_0 >>$@

out.lav: out.blastz
	cp out.blastz out.lav

out.ov.ps: out.blastz exons colors
	overview -e exons -u colors -- out.blastz > $@
