#include <stdio.h>
#include <string.h>
#include <stdlib.h>

const char rcsid[] =
"$Id: mkpipcmd.c,v 1.12 2002/07/26 21:20:50 schwartz Exp $";

#define TMP "./"
const char CLIPPED_ALIGN[] = TMP "pip_clip_align";
const char CLIPPED_TAGS[] = TMP "pip_clip_tags";
const char CLIPPED_ULAY[] = TMP "pip_clip_ulay";
const char CLIP_ALIGN_PROG[] = "clip_align";
const char CLIP_TAG_PROG[] = "clip_tag";
const char CLIP_ULAY_PROG[] = "clip_ulay";
const char EPS_NAME[] = "eps";

int get_seqlen(char *align_file);

void fatal(char *msg)
{
	fprintf(stderr, "%s\n", msg);
	exit(1);
}

int main(int argc, char **argv)
{
	char *header, *alignment, *tag_file, *underlay_file;
	int four_lines = 0, bp_on_page;
	long to, begin, end;
	float y;
	int n;
	int page;

	enum {
		PPI = 2500, /* pixels per inch in each pip element */
		Y0  = 1500, /* 50% mark */
		Y1  = 3000  /* 100% mark */
	}; 

	if (argc == 6 && !strcmp(argv[5], "4lines")) {
		four_lines =  1;
		--argc;
	}
	if (argc != 5)
	    fatal("alignment header_file tag_file underlay_file [4lines]");

	alignment = argv[1];
	header = argv[2];
	tag_file = argv[3];
	underlay_file = argv[4];
	bp_on_page = (four_lines ? 80000 : 100000);

	to = get_seqlen(alignment);

	page = n = 0;
	for (begin = 0; begin < to; begin = end) {
		if (begin % bp_on_page == 0)
			y = 6.0;
		end = begin + 20000;
		if (end > to)
			end = to;

		printf("%s %s %d %d >%s\n",
		    CLIP_ALIGN_PROG, alignment, begin, end, CLIPPED_ALIGN);
		printf("%s %s %d %d >%s\n",
		    CLIP_TAG_PROG, tag_file, begin, end, CLIPPED_TAGS);
		printf("%s %d %d <%s >%s\n",
		    CLIP_ULAY_PROG, begin, end, underlay_file, CLIPPED_ULAY);

		printf("pmps -Y %d -w 1 -align %s",  Y1, CLIPPED_ALIGN);
		printf(" -utag %s", CLIPPED_ULAY);
		printf(" -htag %s", CLIPPED_TAGS);
		printf(" -dot -nb");
		printf(" -ppi=%d", PPI);
		printf(" -vt=25 -ht=1000 -tl=2000 -landscape");
		printf(" -noprocset");
		printf(" -strictlabels -percent");
		printf(" -ticfontsize=9 -labelfontsize=10 -bannerfontsize=11");
		printf(" -y=%3.1f %d %d %d %d", y, begin, end, Y0, Y1);
		printf(" > %s.%05d.%05d\n", EPS_NAME, page, n);
		y -= (four_lines ? 2.0 : 1.5 );
		n += 1;
		if (end < to && (end % bp_on_page) == 0)
			page += 1;
	}
	return 0;
}

int get_seqlen(char *align_file)
{
	FILE *fp;
	char buf[500], *p, *q;
	int i, j;

	if ((fp = fopen(align_file, "r")) == 0)
	    fatal("fopen align_file");

	while (fgets(buf, 500, fp)) {
		if (strncmp(buf, "s {", 3))
			continue;
		if (fgets(buf, 500, fp) == NULL)
			break;
		if ((q=strchr(buf,'"')) == NULL || (p=strchr(q+1,'"')) == NULL)
			break;
		if (sscanf(p+1, "%d %d", &i, &j) != 2 || j <= 0)
			break;
		fclose(fp);
		return j;
	}
	fclose(fp);
	fprintf(stderr,"could not find sequence length in alignment file.");
	return 1;
}
