#include "refine.h"

static void pr_align(align_ptr A, int nRow)
{
	align_ptr ap;
	int k;

	for (k = 0; k < nRow; ++k) {
		for (ap = A; ap; ap = ap->next)
			fputc(ap->col[k], stderr);
		fputc('\n', stderr);
	}
}

static void free_all(align_ptr a, piece_ptr P)
{
	align_ptr b;
	int i;

	for ( ; a; a = b) {
		b = a->next;
		free(a->col);
		free(a);
	}
	for (i = 0; i < 100; ++i)
		free(P[i].seq);
}

int main(int argc, char **argv)
{
	FILE *fp;
	int i, j, k, K, M, r, nP;
	char buf[500];
	uchar **AL;
	align_ptr A, ap, bp;
	ss_t ss;
	int gop[256], gtype[128];
	piece P[100];

	argv0 = "robin.driver";
	if (argc != 2)
		fatal("args = alignment_file");

	for (i = 0; i < 100; ++i)
		P[i].seq = (uchar *)ckalloc(1000*sizeof(uchar));

	fp = ckopen(argv[1], "r");

	fgets(buf, 500, fp);
	M = strlen(buf) - 1;	/* don't count newline */
	AL = (uchar **)ckalloc(M*sizeof(uchar *)) - 1;
	for (i = 1; i <= M; ++i) {
		AL[i] = ckalloc(10*sizeof(uchar));
		AL[i][0] = buf[i-1];
	}
	for (K = 1; fgets(buf, 500, fp); ++K) {
		if (strlen(buf) - 1 != (unsigned)M)
			fatal("Alignment has rows of unequal length.");
		for (i = 1; i <= M; ++i)
			AL[i][K] = buf[i-1];
	}

	ap = A = (align_ptr) ckalloc(sizeof(*A));
	ap->col = AL[1];
	ap->colno = 1;
	for (i = 2; i <= M; ++i) {
		bp = (align_ptr) ckalloc(sizeof(*A));
		bp->col = AL[i];
		bp->colno = i;
		bp->prev = ap;
		ap = ap->next = bp;
	}
	ap->next = NULL;
	A->prev = ap;
	free(AL+1);
	
	fprintf(stderr, "Initial alignment:\n");
	pr_align(A, K);

	for (nP = k = 0; k < K; ++k) {
		for (ap = A; ap; ap = ap->next) {
			if (isalpha(ap->col[k]) &&
			    (ap == A || (ap->prev)->col[k] == FREE)) {
				P[nP].firstcol = ap;
				P[nP].row = k;
			}
			if (isalpha(ap->col[k]) &&
			    (ap->next == NULL || (ap->next)->col[k] == FREE))
				P[nP++].lastcol = ap;
		}
	}

	for (i = 0; i < nP; ++i) {
		for (j = 0, r = P[i].row, ap = P[i].firstcol;
		    ap != P[i].lastcol->next; ap = ap->next)
			if (isalpha(ap->col[r]))
				P[i].seq[j++] = ap->col[r];
		P[i].len = j;
	}

/*
	for (i = 0; i < nP; ++i) {
		fprintf(stderr,"piece %d:row = %d, columns %d-%d, length %d:\n ",
			i, P[i].row, P[i].firstcol->colno, P[i].lastcol->colno,
			P[i].len);
		for (j = 0; j < P[i].len; ++j)
			fputc(P[i].seq[j],stderr);
		fputc('\n',stderr);
	}
*/
		
	for (i = 0; i < 128; ++i)
		for (j = 0; j < 128; ++j) {
			ss[i][j] = 0;
			if (strchr("ACGT",(uchar)i) && strchr("ACGT",(uchar)j))
				ss[i][j] = (i==j ? 10 : -10);
		}


	for (i = 0; i < 128; ++i)
		ss[i][DASH] = ss[DASH][i] = -1;
	ss[DASH][DASH] = 0;
	for (i = 0; i < 128; ++i)
		ss[i][FREE] = ss[FREE][i] = 0;
	gap_costs(gop, gtype, 10);

	A = robin(A, K, P, nP, 10, ss, gop, gtype, 2, 2);
	fprintf(stderr, "score = %d\n", align_score(A, K, ss, gop, gtype));

	free_all(A, P);

	return 0;
}
